(******************************************************************
**
** Based on work from:
** Copyright (c) 1999, Rangel Gustavo Reale (hitnrun@geocities.com)
**
**
** MySQLWrQu - is a Insert, Update and other SQL-commands
** ( WriteQuery )
** The "DoQuery" function gives the change-result (affected rows) to
**
** Ch. Friedrich 16.6.2001
**
** Dependencies:
**      libmysqlclient.so
**
*******************************************************************)
unit MySQLWrQu;

interface

uses
     QDialogs, SysUtils, Classes,mysql_types;


type

  TServerInfo = RECORD
    Host : String[40];
    User : String[40];
    Database : String[40];
  end;

  TSvInfo = procedure (Sender : TObject; data : TServerInfo) of Object;

  TMySqlWrQuery = class(TComponent)
  private
    FSQL: TStrings;
    FDatabase: string;
    FHost: string;
    FPort: Integer;
    FUser: string;
    FPassword: string;
    FConnected:boolean;
    FQueryStr: string;
    // MySQL data
    FMYSQL: PMYSQL;
    FMYSQLRES: PMYSQL_RES;

    FServerInfo: string;
    FHostInfo: string;

    FSvInfo : TServerInfo;
    FOnSvInfo : TSvInfo;

    FAffectedRows: Integer;
    FLastInsertID: Integer;
    FOnDisconnect  : TNotifyEvent;

  protected

    procedure SetSQL(const Value: TStrings);
    procedure DatabaseError(SError:string);

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function GetServerStatus: string;
    function GetClientInfo: string;
    procedure ExecSQL;
    procedure DoOpen;
    procedure Connect;
    procedure DisConnect;
    procedure DoClose;
    function SelectDatabase(const SelDatabase: string):boolean;
    function CreateDatabase(const CrDatabase: string):boolean;
    function DoQuery:integer;
    function SendPing:boolean;
    procedure QueryExec;
    procedure DoGetResult;

  published
    property SQL: TStrings read FSQL write SetSQL;
    property Database: string read FDatabase write FDatabase;
    property Host: string read FHost write FHost;
    property Port: Integer read FPort write FPort;
    property User: string read FUser write FUser;
    property Password: string read FPassword write FPassword;
    property QueryStr: string read FQueryStr write FQueryStr;

    property ServerInfo: string read FServerInfo;
    property ClientInfo: string read GetClientInfo;
    property HostInfo: string read FHostInfo;
    property AffectedRows: Integer read FAffectedRows;
    property LastInsertID: Integer read FLastInsertID;
    property ServerStatus: string read GetServerStatus;
    property Connected: boolean read FConnected;

    property OnServerConnect : TSvInfo read FOnSvInfo write FOnSvInfo;
    property OnDisconnect  : TNotifyEvent read FOnDisconnect write FOnDisconnect;

  end;


procedure Register;

implementation

{ TMySqlWrQuery }

const
  MYSQLDLL = 'libmysqlclient.so';

function mysql_init(mysql: PMYSQL): PMYSQL; cdecl; external MYSQLDLL;
function mysql_connect(mysql: PMYSQL; host, user, passwd: PChar): PMYSQL; cdecl; external MYSQLDLL;
function mysql_real_connect(mysql: PMYSQL; host, user, passwd: PChar; port: integer;
   unix_socket: PChar; clientflag: integer): PMYSQL; cdecl; external MYSQLDLL;
procedure mysql_close(sock: PMYSQL); cdecl; external MYSQLDLL;
function mysql_select_db(mysql: PMYSQL; db: PChar): Integer; cdecl; external MYSQLDLL;
function mysql_create_db(mysql: PMYSQL; DB: PChar): Integer; cdecl; external MYSQLDLL;
procedure mysql_free_result(result: PMYSQL_RES); cdecl; external MYSQLDLL;
function mysql_affected_rows(mysql: PMYSQL):my_ulonglong; cdecl; external MYSQLDLL;
function mysql_error(mysql: PMYSQL): PChar; cdecl; external MYSQLDLL;
function mysql_insert_id(mysql: PMYSQL):my_ulonglong; cdecl; external MYSQLDLL;
function mysql_query(mysql: PMYSQL; q: PChar): Integer; cdecl; external MYSQLDLL;
function mysql_store_result(mysql: PMYSQL): PMYSQL_RES; cdecl; external MYSQLDLL;
function mysql_ping(mysql: PMYSQL): Integer; cdecl; external MYSQLDLL;
function mysql_stat(mysql: PMYSQL): PChar; cdecl; external MYSQLDLL;
function mysql_get_server_info(mysql: PMYSQL): PChar; cdecl; external MYSQLDLL;
function mysql_get_client_info: PChar; cdecl; external MYSQLDLL;
function mysql_get_host_info(mysql: PMYSQL): PChar; cdecl; external MYSQLDLL;



constructor TMySqlWrQuery.Create(AOwner: TComponent);
begin
     inherited; // Create;
     FSQL := TStringList.Create;
     FHost := 'localhost';
     FPort := 0;
     FUser := 'root';
     FPassword := '';
     FDatabase :='mysql';
     FAffectedRows := 0;
     FLastInsertID := -1;
     FConnected:=false;
     FMYSQL := nil;
     FMYSQLRES := nil;
end;

destructor TMySqlWrQuery.Destroy;
begin
     FSQL.Free;
     inherited Destroy;
end;

procedure TMySqlWrQuery.DatabaseError(SError:string);
begin
   showmessage(SError);
end;


procedure TMySqlWrQuery.SetSQL(const Value: TStrings);
begin
   FSQL.Assign(Value);
end;

procedure TMySqlWrQuery.DisConnect;
begin
 DoClose;
end;


procedure TMySqlWrQuery.DoClose;
begin
   try
      if FMYSQLRES <> nil then
         mysql_free_result(FMYSQLRES);
      if FMYSQL <> nil then
         mysql_close(FMYSQL);
      FConnected:=false;
      if Assigned(FOnDisconnect) then FOnDisconnect(Self);
   finally
      FMYSQLRES := nil;
      FMYSQL := nil;
      FConnected:=false;
   end;
end;


procedure TMySqlWrQuery.Connect;
begin
 DoOpen;
end;


procedure TMySqlWrQuery.DoOpen;
begin
   FConnected:=false;
   mysql_init(FMYSQL);
   FMYSQL := mysql_connect(FMYSQL, PChar(FHost), PChar(FUser), PChar(FPassword));
   if FMYSQL = nil then
      DatabaseError('Error connecting to MySQL server')
   else begin
     FConnected:=true;
     if FDatabase <> '' then
      if mysql_select_db(FMYSQL, PChar(FDatabase)) <> 0 then
         DatabaseError(mysql_error(FMYSQL));
     FSvInfo.Host:=FHost;
     FSvInfo.User:=FUser;
     FSvInfo.Database:=FDatabase;
     if Assigned(FOnSvInfo) then FOnSvInfo(Self,FSvInfo);
   end;
end;


procedure TMySqlWrQuery.ExecSQL;
begin
   try
      DoOpen;
      try
         DoQuery;
      finally
         DoClose;
      end;
   finally
      FMYSQLRES := nil;
      FMYSQL := nil;
   end;
end;


function  TMySqlWrQuery.SelectDatabase(const SelDatabase: string):boolean;
begin
  Result:=false; // default Error
  FDatabase := trim(SelDatabase);
  if FConnected and (FDatabase <> '') then
    if mysql_select_db(FMYSQL, PChar(FDatabase)) <> 0 then
      DatabaseError(mysql_error(FMYSQL))
    else Result:=true;
end;


function TMySqlWrQuery.CreateDatabase(const CrDatabase: string):boolean;
begin
  Result:=false; // default Error
  if FConnected and (trim(CrDatabase) <> '') then
    if mysql_create_db(FMYSQL, PChar(CrDatabase)) <> 0 then
           DatabaseError(mysql_error(FMYSQL))
    else Result:=true;
end;


procedure TMySqlWrQuery.QueryExec;
begin
   if mysql_query(FMYSQL, PChar(FQueryStr)) <> 0 then
      DatabaseError(mysql_error(FMYSQL))
end;


function TMySqlWrQuery.DoQuery:integer;
var
  Query: string;
begin
   FAffectedRows := 0;
   Result:=0;
   Query := FSQL.GetText;
   if mysql_query(FMYSQL, PChar(Query)) <> 0 then
      DatabaseError(mysql_error(FMYSQL))
   else begin
     FAffectedRows := mysql_affected_rows(FMYSQL);
     FLastInsertID := mysql_insert_id(FMYSQL);
     Result:=FAffectedRows;
   end;
end;


procedure TMySqlWrQuery.DoGetResult;
begin
   FMYSQLRES := mysql_store_result(FMYSQL);
   if FMYSQLRES = nil then
      DatabaseError(mysql_error(FMYSQL));
   FAffectedRows := mysql_affected_rows(FMYSQL);
end;

// Return Values: True if the server is alive.
function TMySqlWrQuery.SendPing:boolean;
begin
  Result := false;
  if FMYSQL = nil then DoOpen;
  if FConnected then
   Result := mysql_ping(FMYSQL)=0;
end;

function TMySqlWrQuery.GetServerStatus: string;
begin
   if FMYSQL = nil then DoOpen;
   Result := mysql_stat(FMYSQL);
end;

function TMySqlWrQuery.GetClientInfo: string;
begin
   Result :=  mysql_get_client_info;
end;


procedure Register;
begin
  RegisterComponents('Data Access', [TMySqlWrQuery]);
end;

end.
