unit mysql_types;

interface

type
  my_bool = Byte;
  gptr    = PChar;
  socket  = Word;
// ENUM - mysql_option
type
  mysql_option = Byte;
const
  MYSQL_OPT_CONNECT_TIMEOUT                  = 0;
  MYSQL_OPT_COMPRESS                         = 1;
  MYSQL_OPT_NAMED_PIPE                       = 2;

// ENUM - mysql_status
type
  mysql_status = Byte;
const
  MYSQL_STATUS_READY                         = 0;
  MYSQL_STATUS_GET_RESULT                    = 1;
  MYSQL_STATUS_USE_RESULT                    = 2;


// STRUCT - MYSQL
const
   MYSQL_ERRMSG_SIZE            = 200;
   NET_READ_TIMEOUT             = 30;			(* Timeout on read *)
   NET_WRITE_TIMEOUT            = 60;			(* Timeout on write *)
   NET_WAIT_TIMEOUT             = 8*60*60;		(* Wait for new query *)

// STRUCT - USED_MEM
type
  PUSED_MEM = ^USED_MEM;
  USED_MEM = record                            (* struct for once_alloc *)
               next: PUSED_MEM;                       (* Next block in use *)
               left: integer;                           (* memory left in block  *)
               size: integer;                           (* size of block *)
             end;

  PERR_PROC = ^ERR_PROC;
  ERR_PROC = procedure;

// STRUCT - MEM_ROOT
type
  PMEM_ROOT = ^MEM_ROOT;
  MEM_ROOT = packed record
    free:          PUSED_MEM;
    used:          PUSED_MEM;
    pre_alloc:     PUSED_MEM;
    min_malloc:    Integer;
    block_size:    Integer;
    error_handler: PERR_PROC;
  end;


// MYSQL_ROW
type
  CAST_MYSQL_ROW = array[0..$FFFF] of PChar;
  MYSQL_ROW = ^PChar;

type
  MYSQL_FIELD_OFFSET = integer;
  my_ulonglong = Int64;

// ENUM - enum_net_type
type
  enum_net_type = Integer;
const
  NET_TYPE_TCPIP                = 0;
  NET_TYPE_SOCKET               = 1;
  NET_TYPE_NAMEDPIPE            = 2;

// ENUM - enum_field_types
type
  enum_field_types = Byte;
const
  FIELD_TYPE_DECIMAL             = 0;
  FIELD_TYPE_TINY                = 1;
  FIELD_TYPE_SHORT               = 2;
  FIELD_TYPE_LONG                = 3;
  FIELD_TYPE_FLOAT               = 4;
  FIELD_TYPE_DOUBLE              = 5;
  FIELD_TYPE_NULL                = 6;
  FIELD_TYPE_TIMESTAMP           = 7;
  FIELD_TYPE_LONGLONG            = 8;
  FIELD_TYPE_INT24               = 9;
  FIELD_TYPE_DATE                = 10;
  FIELD_TYPE_TIME                = 11;
  FIELD_TYPE_DATETIME            = 12;
  FIELD_TYPE_YEAR                = 13;
  FIELD_TYPE_NEWDATE             = 14;
  FIELD_TYPE_ENUM                = 247;
  FIELD_TYPE_SET                 = 248;
  FIELD_TYPE_TINY_BLOB           = 249;
  FIELD_TYPE_MEDIUM_BLOB         = 250;
  FIELD_TYPE_LONG_BLOB           = 251;
  FIELD_TYPE_BLOB                = 252;
  FIELD_TYPE_VAR_STRING          = 253;
  FIELD_TYPE_STRING              = 254;
  FIELD_TYPE_CHAR                = FIELD_TYPE_TINY;		(* For compability *)
  FIELD_TYPE_INTERVAL            = FIELD_TYPE_ENUM;	        (* For compability *)

  // STRUCT - st_mysql_options
type
  st_mysql_options = record
                       connect_timeout: integer;
                       compress, named_pipe: my_bool;
                     end;


// STRUCT - MYSQL_FIELD
type
  PMYSQL_FIELD = ^MYSQL_FIELD;
  MYSQL_FIELD = record
                  name: PChar;                        (* Name of column *)
                  table: PChar;                       (* Table of column if column was a field *)
                  def: PChar;                         (* Default value (set by mysql_list_fields) *)
                  fieldtype: enum_field_types;        (* Type of field. Se mysql_com.h for types *)
                  length: integer;                      (* Width of column *)
                  max_length: integer;                  (* Max width of selected set *)
                  flags: integer;                       (* Div flags *)
                  decimals: integer;                    (* Number of decimals in field *)
                end;


type

  NET = packed record
    vio:           Pointer;
    fd:            Integer;
    fcntl:         Integer;
    buff:          PChar;
    buff_end:      PChar;
    write_pos:     PChar;
    read_pos:      PChar;
    last_error:    array[01..MYSQL_ERRMSG_SIZE] of Char;
    last_errno:    Integer;
    max_packet:    Integer;
    timeout:       Integer;
    pkt_nr:        Integer;
    error:         Char;
    return_errno:  my_bool;
    compress:      my_bool;
    no_send_ok:    my_bool;
    remain_in_buf: LongInt;
    length:        LongInt;
    buf_length:    LongInt;
    where_b:       LongInt;
    return_status: Pointer;
    reading_or_writing: Char;
    save_char:     Char;
  end;

  PMYSQL = ^MYSQL;
  MYSQL = record
    net:            NET;
    connector_fd:    PChar;
    host:            PChar;
    user:            PChar;
    passwd:          PChar;
    unix_socket:     PChar;
    server_version:  PChar;
    host_info:       PChar;
    info:            PChar;
    db:              PChar;
    port:            Integer;
    client_flag:     Integer;
    server_capabilities: Integer;
    protocol_version: Integer;
    field_count:     Integer;
    server_status:   Integer;
    thread_id:       LongInt;
    affected_rows:   my_ulonglong;
    insert_id:       my_ulonglong;
    extra_info:      my_ulonglong;
    packet_length:   LongInt;
    status:          mysql_status;
    fields:          PMYSQL_FIELD;
    field_alloc:     MEM_ROOT;
    free_me, reconnect: my_bool;
    options:         st_mysql_options; //_mysql_options;
    scramble_buff:   array[0..8] of Char;
    charset:         PChar;
    server_language: Integer;
  end;

// STRUCT - MYSQL_ROWS
type
  PMYSQL_ROWS = ^MYSQL_ROWS;
  MYSQL_ROWS = record
                 next: PMYSQL_ROWS;
                 data: MYSQL_ROW;
               end;
  MYSQL_ROW_OFFSET = MYSQL_ROWS;

// STRUCT - MYSQL_DATA
type
  PMYSQL_DATA = ^MYSQL_DATA;
  MYSQL_DATA = record
                 rows: my_ulonglong;
                 fields: integer;
                 data: PMYSQL_ROWS;
                 alloc: MEM_ROOT;
               end;


// STRUCT - MYSQL_RES
type
  PMYSQL_RES = ^MYSQL_RES;
  MYSQL_RES = record
                row_count: my_ulonglong;
                field_count, current_field: integer;
                fields: PMYSQL_FIELD;
                data: PMYSQL_DATA;
                data_cursor: PMYSQL_ROWS;
                field_alloc: MEM_ROOT;
                row: MYSQL_ROW;			(* If unbuffered read *)
                current_row: MYSQL_ROW;		(* buffer to current row *)
                lengths: Pinteger;		(* column lengths of current row *)
                handle: PMYSQL;		        (* for unbuffered reads *)
                eof: my_bool;			(* Used my mysql_fetch_row *)
              end;


implementation

end.
 